//=============================================================================================================================
//
// Copyright (c) 2015-2018 VisionStar Information Technology (Shanghai) Co., Ltd. All Rights Reserved.
// EasyAR is the registered trademark or trademark of VisionStar Information Technology (Shanghai) Co., Ltd in China
// and other countries for the augmented reality technology developed by VisionStar Information Technology (Shanghai) Co., Ltd.
//
//=============================================================================================================================

#import "ViewController.h"

#import "OpenGLView.h"

@implementation ViewController {
    OpenGLView *glView;
    BOOL started;
    NSString * url;
}

- (void)loadView {
    self->glView = [[OpenGLView alloc] initWithFrame:CGRectZero];
    self->started = NO;
    self->url = @"";
    self.view = self->glView;

    UIButton *button_start = [UIButton buttonWithType:UIButtonTypeSystem];
    button_start.frame = CGRectMake(0, [UIScreen mainScreen].bounds.size.height-50, 70, 50);
    [button_start setTitle:@"Capture" forState:UIControlStateNormal];
    button_start.backgroundColor = [UIColor yellowColor];
    [button_start addTarget:self action:@selector(button_start_clicked) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:button_start];

    UIButton *button_stop = [UIButton buttonWithType:UIButtonTypeSystem];
    button_stop.frame = CGRectMake([UIScreen mainScreen].bounds.size.width-70, [UIScreen mainScreen].bounds.size.height-50, 70, 50);
    [button_stop setTitle:@"Stop" forState:UIControlStateNormal];
    button_stop.backgroundColor = [UIColor yellowColor];
    [button_stop addTarget:self action:@selector(button_stop_clicked) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:button_stop];
}

+ (NSString*)prepareURL {
    static long size = 0;
    size ++ ;
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDir = [paths objectAtIndex:0];
    NSString *album = [docDir stringByAppendingPathComponent:@"album"];
    if (![[NSFileManager defaultManager] fileExistsAtPath:album]) {
        NSLog(@"first run");
        [[NSFileManager defaultManager] createDirectoryAtPath:album withIntermediateDirectories:YES attributes:nil error:nil];
        NSLog(@"%@",album);
    }
    double timenumber = [NSDate date].timeIntervalSince1970;
    NSLog(@"timenumber %lf",timenumber);
    NSString *time = [NSString stringWithFormat:@"%lf",timenumber];
    time = [time stringByReplacingOccurrencesOfString:@"." withString:@"mm"];
    NSLog(@"%@",time);
    NSString *videoString = [NSString stringWithFormat:@"%@number%08ld.mp4",time,size];
    NSString *path = [NSString stringWithFormat:@"%@/%@",album,videoString];
    NSLog(@"path %@",path);
    return path;
}

- (void)button_start_clicked {
    if (started) {
        return;
    }
    [self->glView requestPermissions:^(easyar_PermissionStatus status, NSString * msg) {
        switch (status) {
            case easyar_PermissionStatus_Denied:
                started = NO;
                [ViewController displayToastWithMessage:@"Permission Denied"];
                break;
            case easyar_PermissionStatus_Error:
                started = NO;
                [ViewController displayToastWithMessage:@"Permission Error"];
                break;
            case easyar_PermissionStatus_Granted:
                url = [ViewController prepareURL];
                {
                    __weak ViewController * weak_self = self;
                    [self->glView startRecording:url callback:^(easyar_RecordStatus status, NSString *value) {
                        if (status == easyar_RecordStatus_OnStopped) {
                            ViewController * strong_self = weak_self;
                            strong_self->started = NO;
                        }
                        NSLog(@"Recorder Callback status: %d, MSG: %@", (int)(status), value);
                    }];
                }
                [ViewController displayToastWithMessage:@"Recording..."];
                break;
            default:
                break;
        }
    }];
    started = YES;
}

- (void)button_stop_clicked {
    if (!started) {
        return;
    }
    [self->glView stopRecording];
    [ViewController displayToastWithMessage:[@"Recorded at " stringByAppendingString:url]];
    started = NO;
}

+ (void)displayToastWithMessage:(NSString *)toastMessage
{
    [[NSOperationQueue mainQueue] addOperationWithBlock:^ {
        UIWindow * keyWindow = [[UIApplication sharedApplication] keyWindow];
        UILabel * toastView = [[UILabel alloc] init];
        toastView.text = toastMessage;
        toastView.textAlignment = NSTextAlignmentCenter;
        toastView.lineBreakMode = NSLineBreakByWordWrapping;
        toastView.numberOfLines = 0;
        toastView.frame = CGRectMake(0.0, 0.0, keyWindow.frame.size.width/2.0, 200.0);
        toastView.layer.cornerRadius = 10;
        toastView.layer.masksToBounds = YES;
        toastView.center = keyWindow.center;

        [keyWindow addSubview:toastView];

        [UIView animateWithDuration: 3.0f
                          delay: 0.0
                        options: UIViewAnimationOptionCurveEaseOut
                     animations: ^{
                         toastView.alpha = 0.0;
                     }
                     completion: ^(BOOL finished) {
                         [toastView removeFromSuperview];
                     }
         ];
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self->glView setOrientation:self.interfaceOrientation];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self->glView start];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self->glView stop];
}

-(void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];
    [self->glView resize:self.view.bounds orientation:self.interfaceOrientation];
}

-(void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    [super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    [self->glView setOrientation:toInterfaceOrientation];
}

@end
