//=============================================================================================================================
//
// Copyright (c) 2015-2018 VisionStar Information Technology (Shanghai) Co., Ltd. All Rights Reserved.
// EasyAR is the registered trademark or trademark of VisionStar Information Technology (Shanghai) Co., Ltd in China
// and other countries for the augmented reality technology developed by VisionStar Information Technology (Shanghai) Co., Ltd.
//
//=============================================================================================================================

#import "OpenGLView.h"
#import "AppDelegate.h"
#import <UIKit/UIKit.h>

#import "helloar.h"

#import <easyar/engine.oc.h>

@interface OpenGLView ()
{
}

- (void)onStartStopClicked:(id)sender;

@end

@implementation OpenGLView {
    BOOL initialized;
    BOOL started;
    UIButton * startStopButton;
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    self->initialized = false;
    self->started = false;
    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
    self.drawableColorFormat = GLKViewDrawableColorFormatRGBA8888;
    self.drawableDepthFormat = GLKViewDrawableDepthFormat24;
    self.drawableStencilFormat = GLKViewDrawableStencilFormat8;
    [self bindDrawable];
    
    startStopButton = [[UIButton alloc] init];
    [startStopButton setTitle:@"Start" forState:UIControlStateNormal];
    startStopButton.contentEdgeInsets = UIEdgeInsetsMake(10, 10, 10, 10);
    [startStopButton sizeToFit];
    [startStopButton setBackgroundColor:[UIColor grayColor]];
    [self addSubview:startStopButton];
    startStopButton.translatesAutoresizingMaskIntoConstraints = NO;
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 9.0f) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wpartial-availability"
        [startStopButton.centerXAnchor constraintEqualToAnchor: self.layoutMarginsGuide.centerXAnchor].active = YES;
        [startStopButton.bottomAnchor constraintEqualToAnchor: self.layoutMarginsGuide.bottomAnchor].active = YES;
#pragma clang diagnostic pop
    }
    
    [startStopButton addTarget:self action:@selector(onStartStopClicked:) forControlEvents:UIControlEventTouchUpInside];
    
    return self;
}

- (void)onStartStopClicked:(id)sender
{
    UIButton * startStopButton = (UIButton *)sender;
    if (started) {
        [startStopButton setTitle:@"Start" forState:UIControlStateNormal];
        stopTracker();
    } else {
        [startStopButton setTitle:@"Stop" forState:UIControlStateNormal];
        startTracker();
    }
    started = !started;
}

- (void)dealloc
{
}

- (void)start
{
    if (initialize()) {
        start();
    }
}

- (void)stop
{
    stop();
    finalize();
}

- (void)resize:(CGRect)frame orientation:(UIInterfaceOrientation)orientation
{
    float scale;
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 8.0f) {
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wpartial-availability"
        scale = [[UIScreen mainScreen] nativeScale];
#pragma clang diagnostic pop
    } else {
        scale = [[UIScreen mainScreen] scale];
    }

    resizeGL(frame.size.width * scale, frame.size.height * scale);
    
    if ([[UIDevice currentDevice].systemVersion floatValue] >= 9.0f) {
    } else {
        if (startStopButton != nil) {
            startStopButton.frame = CGRectMake(frame.size.width / 2 - startStopButton.frame.size.width / 2, frame.size.height - startStopButton.frame.size.height, startStopButton.frame.size.width, startStopButton.frame.size.height);
        }
    }
}

- (void)drawRect:(CGRect)rect
{
    if (!initialized) {
        initGL();
        initialized = YES;
    }
    render();
}

- (void)setOrientation:(UIInterfaceOrientation)orientation
{
    switch (orientation)
    {
        case UIInterfaceOrientationPortrait:
            [easyar_Engine setRotation:270];
            break;
        case UIInterfaceOrientationPortraitUpsideDown:
            [easyar_Engine setRotation:90];
            break;
        case UIInterfaceOrientationLandscapeLeft:
            [easyar_Engine setRotation:180];
            break;
        case UIInterfaceOrientationLandscapeRight:
            [easyar_Engine setRotation:0];
            break;
        default:
            break;
    }
}

@end
